#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <string.h>
#include <stdint.h>
#include <inttypes.h>

int announcement(char *prog)
{
    char buf[1024];

    snprintf(buf,sizeof(buf),"%s",prog);
    return printf("%s: ok (%p)\n",buf,announcement);
}

int main(int argc, char * argv[])
{
    return !(announcement(argv[0]) > 0);
}
